@@ -3,6 +3,7 @@
 
 #include <stdint.h>
 #include <getopt.h>
+#include <stdbool.h>
 
 #define COMMAND_MAX_LEN 1024
 
@@ -26,11 +27,13 @@ enum {
     opt_command_fd,
     opt_comment,
     opt_compress_algo,
+    opt_default_recipient,
     opt_default_recipient_self,
     opt_digest_algo,
     opt_disable_cipher_algo,
     opt_disable_mdc,
     opt_disable_pubkey_algo,
+    opt_display,
     opt_display_charset,
     opt_emit_version,
     opt_enable_progress_filter,
@@ -98,6 +101,7 @@ enum {
     opt_verify,
     opt_verify_options,
     opt_version,
+    opt_with_colons,
     opt_with_fingerprint,
     opt_with_keygrip,
     opt_yes
@@ -206,12 +210,41 @@ static const int gpg_allowed_options[] = {
     opt_verify,
     opt_verify_options,
     opt_version,
+    opt_with_colons,
     opt_with_fingerprint,
     opt_with_keygrip,
     opt_yes,
     0
 };
 
+/*
+ * Options that define command to perform. There can be only one of those, and
+ * the command define how non-option arguments are interpreted (either file
+ * path, or user id).
+ */
+struct gpg_command_opt {
+    int opt;
+    bool userid_args;
+};
+static const struct gpg_command_opt gpg_commands[] = {
+    {'K', true},
+    {'b', false},
+    {'c', false},
+    {'d', false},
+    {'e', false},
+    {'k', true},
+    {'s', false},
+    {opt_clearsign, false},
+    {opt_export, true},
+    {opt_export_ssh, true},
+    {opt_fingerprint, true},
+    {opt_list_config, false},
+    {opt_list_sigs, true},
+    {opt_store, false},
+    {opt_verify, false},
+    {0, false},
+};
+
 static const char gpg_short_options[] = "bacdekKnN:qr:R:stu:";
 
 static const struct option gpg_long_options[] = {
@@ -227,12 +260,14 @@ static const struct option gpg_long_options[] = {
     {"comment", 1, 0, opt_comment},
     {"compress-algo", 1, 0, opt_compress_algo},
     {"decrypt", 0, 0, 'd'},
+    {"default-recipient", 1, 0, opt_default_recipient},
     {"default-recipient-self", 0, 0, opt_default_recipient_self},
     {"detach-sign", 0, 0, 'b'},
     {"digest-algo", 1, 0, opt_digest_algo},
     {"disable-cipher-algo", 1, 0, opt_disable_cipher_algo},
     {"disable-mdc", 0, 0, opt_disable_mdc},
     {"disable-pubkey-algo", 1, 0, opt_disable_pubkey_algo},
+    {"display", 1, 0, opt_display},
     {"display-charset", 1, 0, opt_display_charset},
     {"dry-run", 0, 0, 'n'},
     {"emit-version", 0, 0, opt_emit_version},
@@ -240,11 +275,10 @@ static const struct option gpg_long_options[] = {
     {"encrypt", 0, 0, 'e'},
     {"encrypt-to", 1, 0, opt_encrypt_to},
     {"exit-on-status-write-error", 0, 0, opt_exit_on_status_write_error},
-    {"export", 2, 0, opt_export},
-    {"export-ssh-key", 1, 0, opt_export_ssh},
+    {"export", 0, 0, opt_export},
+    {"export-ssh-key", 0, 0, opt_export_ssh},
     {"fingerprint", 0, 0, opt_fingerprint},
     {"fixed-list-mode", 0, 0, opt_fixed_list_mode},
-    {"fixed-list-mode", 0, 0, opt_fixed_list_mode},
     {"force-mdc", 0, 0, opt_force_mdc},
     {"force-v3-sigs", 0, 0, opt_force_v3_sigs},
     {"force-v4-certs", 0, 0, opt_force_v4_certs},
@@ -252,12 +286,12 @@ static const struct option gpg_long_options[] = {
     {"hidden-encrypt-to", 1, 0, opt_hidden_encrypt_to},
     {"hidden-recipient", 1, 0, 'R'},
     {"list-config", 0, 0, opt_list_config},
-    {"list-keys", 2, 0, 'k'},
+    {"list-keys", 0, 0, 'k'},
     {"list-only", 0, 0, opt_list_only},
     {"list-options", 1, 0, opt_list_options},
-    {"list-public-keys", 2, 0, 'k'},
-    {"list-secret-keys", 2, 0, 'K'},
-    {"list-sigs", 2, 0, opt_list_sigs},
+    {"list-public-keys", 0, 0, 'k'},
+    {"list-secret-keys", 0, 0, 'K'},
+    {"list-sigs", 0, 0, opt_list_sigs},
     {"local-user", 1, 0, 'u'},
     {"logger-fd", 1, 0, opt_logger_fd},
     {"keyid-format", 1, 0, opt_keyid_format},
@@ -293,10 +327,10 @@ static const struct option gpg_long_options[] = {
     {"rfc1991", 0, 0, opt_rfc1991},
     {"rfc2440", 0, 0, opt_rfc2440},
     {"rfc4880", 0, 0, opt_rfc4880},
-    {"s2k-cipher-algo", 0, 0, opt_s2k_cipher_algo},
-    {"s2k-count", 0, 0, opt_s2k_count},
-    {"s2k-digest-algo", 0, 0, opt_s2k_digest_algo},
-    {"s2k-mode", 0, 0, opt_s2k_mode},
+    {"s2k-cipher-algo", 1, 0, opt_s2k_cipher_algo},
+    {"s2k-count", 1, 0, opt_s2k_count},
+    {"s2k-digest-algo", 1, 0, opt_s2k_digest_algo},
+    {"s2k-mode", 1, 0, opt_s2k_mode},
     {"sender", 1, 0, opt_sender},
     {"sig-notation", 1, 0, opt_sig_notation},
     {"sign", 0, 0, 's'},
@@ -318,7 +352,7 @@ static const struct option gpg_long_options[] = {
     {"verify-options", 1, 0, opt_verify_options},
     {"verbose", 0, 0, 'v'},
     {"version", 0, 0, opt_version},
-    {"with-colons", 0, 0, 'k'},
+    {"with-colons", 0, 0, opt_with_colons},
     {"with-fingerprint", 0, 0, opt_with_fingerprint},
     {"with-keygrip", 0, 0, opt_with_keygrip},
     {"yes", 0, 0, opt_yes},
