@@ -434,16 +434,31 @@ class TC_10_Thunderbird(SplitGPGBase):
 
         # run as root to not deal with /var/mail permission issues
         self.frontend.run(
-            'touch /var/mail/user; chown user /var/mail/user', user='root',
+            'touch /var/mail/user; chown user:user /var/mail/user', user='root',
             wait=True)
+
+        # SMTP configuration
         self.smtp_server = self.frontend.run(
             'python3 /usr/lib/qubes-gpg-split/test_smtpd.py',
             user='root', passio_popen=True)
 
+        # IMAP configuration
+        self.imap_pw = "pass"
+        self.frontend.run(
+            'echo "mail_location=mbox:~/Mail:INBOX=/var/mail/%u" |\
+                sudo tee /etc/dovecot/conf.d/100-mail.conf', wait=True)
+        self.frontend.run('sudo systemctl restart dovecot', wait=True)
+        self.frontend.run( # set a user password because IMAP needs one for auth
+            'sudo usermod -p `echo "{}" | openssl passwd --stdin` user'\
+                .format(self.imap_pw),
+            wait=True)
+
+        self.setup_tb_profile(setup_openpgp=True)
+
         p = self.frontend.run(
             'PYTHONPATH=$HOME/dogtail LC_ALL=C.UTF-8 '
-            'python3 {} --tbname={} setup 2>&1'.format(
-                self.scriptpath, self.tb_name),
+            'python3 {} --tbname={} --profile {} --imap_pw {} setup 2>&1'.format(
+                self.scriptpath, self.tb_name, self.profile_dir, self.imap_pw),
             passio_popen=True)
         (stdout, _) = p.communicate()
         assert p.returncode == 0, 'Thunderbird setup failed: {}'.format(
@@ -457,12 +472,66 @@ class TC_10_Thunderbird(SplitGPGBase):
         del self.smtp_server
         super(TC_10_Thunderbird, self).tearDown()
 
+    def setup_tb_profile(self, setup_openpgp):
+        """SplitGPG Thunderbird Test Account Configuration
+
+        Originally generated by running thunderbird for the first time
+        and taking from ~/.thunderbird/<PROFILE>.default/prefs.js all
+        the relevant settings. Then adding the opengpg settings.
+        """
+
+        profile_base = """
+user_pref("mail.accountmanager.accounts", "account1");
+user_pref("mail.accountmanager.defaultaccount", "account1");
+user_pref("mail.account.account1.identities", "id1");
+user_pref("mail.account.account1.server", "server1");
+user_pref("mail.identity.id1.fullName", "user");
+user_pref("mail.identity.id1.useremail", "user@localhost");
+user_pref("mail.identity.id1.smtpServer", "smtp1");
+user_pref("mail.identity.id1.compose_html", false);
+"""
+        imap_server = """
+user_pref("mail.server.server1.userName", "user");
+user_pref("mail.server.server1.hostname", "localhost");
+user_pref("mail.server.server1.login_at_startup", true);
+user_pref("mail.server.server1.name", "user@localhost");
+user_pref("mail.server.server1.type", "imap");
+user_pref("mail.server.server1.port", 143);
+"""
+        smtp_server = """
+user_pref("mail.smtpservers", "smtp1");
+user_pref("mail.smtp.defaultserver", "smtp1");
+user_pref("mail.smtpserver.smtp1.username", "user");
+user_pref("mail.smtpserver.smtp1.hostname", "localhost");
+user_pref("mail.smtpserver.smtp1.port", 8025);
+user_pref("mail.smtpserver.smtp1.authMethod", 3); // no auth
+user_pref("mail.smtpserver.smtp1.try_ssl", 0);    // no encryption
+"""
+        open_pgp = """
+user_pref("mail.openpgp.allow_external_gnupg", true);
+user_pref("mail.openpgp.alternative_gpg_path", "/usr/bin/qubes-gpg-client-wrapper");
+"""
+        self.profile_dir = "$HOME/.thunderbird/qubes.default"
+        user_js_path = self.profile_dir + "/user.js"
+
+        user_js = profile_base + imap_server + smtp_server
+        if setup_openpgp:
+            user_js += open_pgp
+
+        self.frontend.run('mkdir -p {}'.format(self.profile_dir),
+                          user='user', wait=True)
+        p = self.frontend.run('cat > ' + user_js_path,
+                          user='user', passio_popen=True)
+        (stdout, _) = p.communicate(user_js.encode())
+        assert p.returncode == 0, 'Thunderbird profile configuration failed: {}'\
+            .format(stdout.decode('ascii', 'ignore'))
+
     def test_000_send_receive_default(self):
         p = self.frontend.run(
             'PYTHONPATH=$HOME/dogtail LC_ALL=C.UTF-8 '
-            'python3 {} --tbname={} send_receive '
+            'python3 {} --tbname={} --profile {} --imap_pw {} send_receive '
             '--encrypted --signed 2>&1'.format(
-                self.scriptpath, self.tb_name),
+                self.scriptpath, self.tb_name, self.profile_dir, self.imap_pw),
             passio_popen=True)
         (stdout, _) = p.communicate()
         self.assertEquals(p.returncode, 0,
@@ -472,10 +541,9 @@ class TC_10_Thunderbird(SplitGPGBase):
     def test_010_send_receive_inline_signed_only(self):
         p = self.frontend.run(
             'PYTHONPATH=$HOME/dogtail LC_ALL=C.UTF-8 '
-            'python3 {} --tbname={} '
-            'send_receive '
+            'python3 {} --tbname={} --profile {} --imap_pw {} send_receive '
             '--encrypted --signed --inline 2>&1'.format(
-                self.scriptpath, self.tb_name),
+                self.scriptpath, self.tb_name, self.profile_dir, self.imap_pw),
             passio_popen=True)
         (stdout, _) = p.communicate()
         self.assertEquals(p.returncode, 0,
@@ -485,9 +553,9 @@ class TC_10_Thunderbird(SplitGPGBase):
     def test_020_send_receive_inline_with_attachment(self):
         p = self.frontend.run(
             'PYTHONPATH=$HOME/dogtail LC_ALL=C.UTF-8 '
-            'python3 {} --tbname={} send_receive '
+            'python3 {} --tbname={} --profile {} --imap_pw {} send_receive '
             '--encrypted --signed --inline --with-attachment 2>&1'.format(
-                self.scriptpath, self.tb_name),
+                self.scriptpath, self.tb_name, self.profile_dir, self.imap_pw),
             passio_popen=True)
         (stdout, _) = p.communicate()
         self.assertEquals(p.returncode, 0,
